<?php

namespace MediaWiki\Skin\Cosmos;

use LogicException;
use MediaWiki\MediaWikiServices;

class LessUtil {
	/** @var array */
	private static $cosmosSettings = [];

	/**
	 * Gets theme settings from Config class
	 *
	 * @return array
	 */
	public static function getCosmosSettings() {
		if ( empty( static::$cosmosSettings ) ) {
			$services = MediaWikiServices::getInstance();
			$themeSettings = $services->getService( 'CosmosConfig' );

			static::$cosmosSettings['banner-background-color'] =
				self::sanitizeColor(
					$themeSettings->getBannerBackgroundColor()
				);

			static::$cosmosSettings['header-background-color'] =
				self::sanitizeColor(
					$themeSettings->getWikiHeaderBackgroundColor()
				);

			static::$cosmosSettings['content-background-color'] =
				self::sanitizeColor(
					$themeSettings->getContentBackgroundColor()
				);

			static::$cosmosSettings['button-background-color'] =
				self::sanitizeColor(
					$themeSettings->getButtonBackgroundColor()
				);

			static::$cosmosSettings['link-color'] =
				self::sanitizeColor(
					$themeSettings->getLinkColor()
				);

			static::$cosmosSettings['toolbar-background-color'] =
				self::sanitizeColor(
					$themeSettings->getToolbarBackgroundColor()
				);

			static::$cosmosSettings['footer-background-color'] =
				self::sanitizeColor(
					$themeSettings->getFooterBackgroundColor()
				);

			foreach ( static::$cosmosSettings as $key => $val ) {
				if ( !empty( $val ) ) {
					static::$cosmosSettings[$key] = trim( $val );
				}
			}
		}

		return static::$cosmosSettings;
	}

	/**
	 * Get normalized color value
	 * @param string $color
	 * @return string
	 */
	public static function sanitizeColor( $color ) {
		$color = trim( strtolower( $color ) );
		return $color;
	}

	/**
	 * Calculates whether currently used theme is light or dark
	 *
	 * @param string $background
	 * @param array|null $cosmosSettings
	 * @return bool
	 */
	public static function isThemeDark( $background, $cosmosSettings = null ) {
		if ( empty( $cosmosSettings ) ) {
			$cosmosSettings = self::getCosmosSettings();
		}

		$backgroundColor = $cosmosSettings[$background];

		// convert RGB to HSL
		list( $hue, $saturation, $lightness ) = self::rgb2hsl( $backgroundColor );

		$isDark = ( $lightness < 0.5 );

		return $isDark;
	}

	/**
	 * Convert RGB colors array into HSL array
	 *
	 * @param string $rgbhex RGB color in hex format (#474646)
	 * @return array HSL set
	 */
	private static function rgb2hsl( $rgbhex ) {
		if ( $rgbhex[0] != '#' ) {
			$rgbhex = self::colorNameToHex( $rgbhex );
		}

		// We need to convert hex shorthand format
		if ( strlen( $rgbhex ) == 4 ) {
			$rgbhex[6] = $rgbhex[3];
			$rgbhex[5] = $rgbhex[3];
			$rgbhex[4] = $rgbhex[2];
			$rgbhex[3] = $rgbhex[2];
			$rgbhex[2] = $rgbhex[1];
		}

		// convert HEX color to rgb values
		// #474646 -> 71, 70, 70
		$rgb = str_split( substr( $rgbhex, 1 ), 2 );
		$rgb = array_map( 'hexdec', $rgb );

		$clrR = ( !empty( $rgb[0] ) ? ( $rgb[0] / 255 ) : 0 );
		$clrG = ( !empty( $rgb[1] ) ? ( $rgb[1] / 255 ) : 0 );
		$clrB = ( !empty( $rgb[2] ) ? ( $rgb[2] / 255 ) : 0 );

		$clrMin = min( $clrR, $clrG, $clrB );
		$clrMax = max( $clrR, $clrG, $clrB );
		$deltaMax = $clrMax - $clrMin;

		$L = ( $clrMax + $clrMin ) / 2;

		if ( $deltaMax == 0 ) {
			$H = 0;
			$S = 0;
		} else {
			if ( $L < 0.5 ) {
				$S = $deltaMax / ( $clrMax + $clrMin );
			} else {
				$S = $deltaMax / ( 2 - $clrMax - $clrMin );
			}
			$deltaR = ( ( ( $clrMax - $clrR ) / 6 ) + ( $deltaMax / 2 ) ) / $deltaMax;
			$deltaG = ( ( ( $clrMax - $clrG ) / 6 ) + ( $deltaMax / 2 ) ) / $deltaMax;
			$deltaB = ( ( ( $clrMax - $clrB ) / 6 ) + ( $deltaMax / 2 ) ) / $deltaMax;
			if ( $clrR == $clrMax ) {
				$H = $deltaB - $deltaG;
			} elseif ( $clrG == $clrMax ) {
				$H = ( 1 / 3 ) + $deltaR - $deltaB;
			} elseif ( $clrB == $clrMax ) {
				$H = ( 2 / 3 ) + $deltaG - $deltaR;
			} else {
				throw new LogicException( 'Unreachable' );
			}
			if ( $H < 0 ) {
				$H += 1;
			}
			if ( $H > 1 ) {
				$H -= 1;
			}
		}

		return [
			$H,
			$S,
			$L
		];
	}

	/**
	 * Convert HTML color name to hex format
	 *
	 * @param string $colorName
	 * @return string
	 */
	public static function colorNameToHex( $colorName ) {
		// standard 147 HTML color names
		$colors = [
			'aliceblue' => '#f0f8ff',
			'antiquewhite' => '#faebd7',
			'aqua' => '#00ffff',
			'aquamarine' => '#7fffd4',
			'azure' => '#f0ffff',
			'beige' => '#f5f5dc',
			'bisque' => '#ffe4c4',
			'black' => '#000000',
			'blanchedalmond ' => '#ffebcd',
			'blue' => '#0000ff',
			'blueviolet' => '#8a2be2',
			'brown' => '#a52a2a',
			'burlywood' => '#deb887',
			'cadetblue' => '#5f9ea0',
			'chartreuse' => '#7fff00',
			'chocolate' => '#d2691e',
			'coral' => '#ff7f50',
			'cornflowerblue' => '#6495ed',
			'cornsilk' => '#fff8dc',
			'crimson' => '#dc143c',
			'cyan' => '#00ffff',
			'darkblue' => '#00008b',
			'darkcyan' => '#008b8b',
			'darkgoldenrod' => '#b8860b',
			'darkgray' => '#a9a9a9',
			'darkgreen' => '#006400',
			'darkgrey' => '#a9a9a9',
			'darkkhaki' => '#bdb76b',
			'darkmagenta' => '#8b008b',
			'darkolivegreen' => '#556b2f',
			'darkorange' => '#ff8c00',
			'darkorchid' => '#9932cc',
			'darkred' => '#8b0000',
			'darksalmon' => '#e9967a',
			'darkseagreen' => '#8fbc8f',
			'darkslateblue' => '#483d8b',
			'darkslategray' => '#2f4f4f',
			'darkslategrey' => '#2f4f4f',
			'darkturquoise' => '#00ced1',
			'darkviolet' => '#9400d3',
			'deeppink' => '#ff1493',
			'deepskyblue' => '#00bfff',
			'dimgray' => '#696969',
			'dimgrey' => '#696969',
			'dodgerblue' => '#1e90ff',
			'firebrick' => '#b22222',
			'floralwhite' => '#fffaf0',
			'forestgreen' => '#228b22',
			'fuchsia' => '#ff00ff',
			'gainsboro' => '#dcdcdc',
			'ghostwhite' => '#f8f8ff',
			'gold' => '#ffd700',
			'goldenrod' => '#daa520',
			'gray' => '#808080',
			'green' => '#008000',
			'greenyellow' => '#adff2f',
			'grey' => '#808080',
			'honeydew' => '#f0fff0',
			'hotpink' => '#ff69b4',
			'indianred' => '#cd5c5c',
			'indigo' => '#4b0082',
			'ivory' => '#fffff0',
			'khaki' => '#f0e68c',
			'lavender' => '#e6e6fa',
			'lavenderblush' => '#fff0f5',
			'lawngreen' => '#7cfc00',
			'lemonchiffon' => '#fffacd',
			'lightblue' => '#add8e6',
			'lightcoral' => '#f08080',
			'lightcyan' => '#e0ffff',
			'lightgoldenrodyellow' => '#fafad2',
			'lightgray' => '#d3d3d3',
			'lightgreen' => '#90ee90',
			'lightgrey' => '#d3d3d3',
			'lightpink' => '#ffb6c1',
			'lightsalmon' => '#ffa07a',
			'lightseagreen' => '#20b2aa',
			'lightskyblue' => '#87cefa',
			'lightslategray' => '#778899',
			'lightslategrey' => '#778899',
			'lightsteelblue' => '#b0c4de',
			'lightyellow' => '#ffffe0',
			'lime' => '#00ff00',
			'limegreen' => '#32cd32',
			'linen' => '#faf0e6',
			'magenta' => '#ff00ff',
			'maroon' => '#800000',
			'mediumaquamarine' => '#66cdaa',
			'mediumblue' => '#0000cd',
			'mediumorchid' => '#ba55d3',
			'mediumpurple' => '#9370d0',
			'mediumseagreen' => '#3cb371',
			'mediumslateblue' => '#7b68ee',
			'mediumspringgreen' => '#00fa9a',
			'mediumturquoise' => '#48d1cc',
			'mediumvioletred' => '#c71585',
			'midnightblue' => '#191970',
			'mintcream' => '#f5fffa',
			'mistyrose' => '#ffe4e1',
			'moccasin' => '#ffe4b5',
			'navajowhite' => '#ffdead',
			'navy' => '#000080',
			'oldlace' => '#fdf5e6',
			'olive' => '#808000',
			'olivedrab' => '#6b8e23',
			'orange' => '#ffa500',
			'orangered' => '#ff4500',
			'orchid' => '#da70d6',
			'palegoldenrod' => '#eee8aa',
			'palegreen' => '#98fb98',
			'paleturquoise' => '#afeeee',
			'palevioletred' => '#db7093',
			'papayawhip' => '#ffefd5',
			'peachpuff' => '#ffdab9',
			'peru' => '#cd853f',
			'pink' => '#ffc0cb',
			'plum' => '#dda0dd',
			'powderblue' => '#b0e0e6',
			'purple' => '#800080',
			'red' => '#ff0000',
			'rosybrown' => '#bc8f8f',
			'royalblue' => '#4169e1',
			'saddlebrown' => '#8b4513',
			'salmon' => '#fa8072',
			'sandybrown' => '#f4a460',
			'seagreen' => '#2e8b57',
			'seashell' => '#fff5ee',
			'sienna' => '#a0522d',
			'silver' => '#c0c0c0',
			'skyblue' => '#87ceeb',
			'slateblue' => '#6a5acd',
			'slategray' => '#708090',
			'slategrey' => '#708090',
			'snow' => '#fffafa',
			'springgreen' => '#00ff7f',
			'steelblue' => '#4682b4',
			'tan' => '#d2b48c',
			'teal' => '#008080',
			'thistle' => '#d8bfd8',
			'tomato' => '#ff6347',
			'turquoise' => '#40e0d0',
			'violet' => '#ee82ee',
			'wheat' => '#f5deb3',
			'white' => '#ffffff',
			'whitesmoke' => '#f5f5f5',
			'yellow' => '#ffff00',
			'yellowgreen' => '#9acd32'
		];

		if ( isset( $colors[$colorName] ) ) {
			return $colors[$colorName];
		} else {
			return $colorName;
		}
	}

	/**
	 * @param string $hex
	 *
	 * @return array with r, g and b keys
	 */
	public static function hexToRgb( string $hex ): array {
		$hex = str_replace( '#', '', $hex );
		$length = strlen( $hex );

		if ( $length == 6 ) {
			$rgb = [
				'r' => hexdec( substr( $hex, 0, 2 ) ),
				'g' => hexdec( substr( $hex, 2, 2 ) ),
				'b' => hexdec( substr( $hex, 4, 2 ) )
			];
		} elseif ( $length == 3 ) {
			$rgb = [
				'r' => hexdec( str_repeat( substr( $hex, 0, 1 ), 2 ) ),
				'g' => hexdec( str_repeat( substr( $hex, 1, 1 ), 2 ) ),
				'b' => hexdec( str_repeat( substr( $hex, 2, 1 ), 2 ) )
			];
		} else {
			$rgb = [ 'r' => 0, 'g' => 0, 'b' => 0 ];
		}

		return $rgb;
	}
}
